@@ -101,26 +101,28 @@ grid.multipanel <- function(x=stats::runif(90), y=stats::runif(90),
 }
 
 grid.show.layout <- function(l, newpage=TRUE,
+                             bg="light grey",
                          cell.border="blue", cell.fill="light blue",
-                         cell.label=TRUE, vp=NULL) {
+                         cell.label=TRUE, label.col="blue",
+                             unit.col="red", vp=NULL) {
   if (!is.layout(l))
     stop("l must be a layout")
   if (newpage)
     grid.newpage()
   if (!is.null(vp))
     pushViewport(vp)
-  grid.rect(gp=gpar(col=NULL, fill="light grey"))
+  grid.rect(gp=gpar(col=NULL, fill=bg))
   vp.mid <- viewport(0.5, 0.5, 0.8, 0.8, layout=l)
   pushViewport(vp.mid)
   grid.rect(gp=gpar(fill="white"))
-  gp.red <- gpar(col="red")
+  gp.red <- gpar(col=unit.col)
   for (i in 1:l$nrow)
     for (j in 1:l$ncol) {
       vp.inner <- viewport(layout.pos.row=i, layout.pos.col=j)
       pushViewport(vp.inner)
       grid.rect(gp=gpar(col=cell.border, fill=cell.fill))
       if (cell.label)
-        grid.text(paste("(", i, ", ", j, ")", sep=""), gp=gpar(col="blue"))
+        grid.text(paste("(", i, ", ", j, ")", sep=""), gp=gpar(col=label.col))
       if (j==1)
         grid.text(as.character(l$heights[i]), gp=gp.red,
               just=c("right", "centre"),
@@ -148,7 +150,11 @@ grid.show.layout <- function(l, newpage=TRUE,
   invisible(vp.mid)
 }
 
-grid.show.viewport <- function(v, parent.layout=NULL, newpage=TRUE, vp=NULL) {
+grid.show.viewport <- function(v, parent.layout=NULL, newpage=TRUE,
+                               border.fill="light grey",
+                               vp.col="blue", vp.fill="light blue",
+                               scale.col="red",
+                               vp=NULL) {
   # if the viewport has a non-NULL layout.pos.row or layout.pos.col
   # AND the viewport has a parent AND the parent has a layout
   # represent the location of the viewport in the parent's layout ...
@@ -161,7 +167,7 @@ grid.show.viewport <- function(v, parent.layout=NULL, newpage=TRUE, vp=NULL) {
                            cell.label=FALSE, newpage=newpage)
     pushViewport(vp.mid)
     pushViewport(v)
-    gp.red <- gpar(col="red")
+    gp.red <- gpar(col=scale.col)
     grid.rect(gp=gpar(col="blue", fill="light blue"))
     at <- grid.pretty(v$xscale)
     grid.xaxis(at=c(min(at), max(at)), gp=gp.red)
@@ -175,7 +181,7 @@ grid.show.viewport <- function(v, parent.layout=NULL, newpage=TRUE, vp=NULL) {
       grid.newpage()
     if (!is.null(vp))
       pushViewport(vp)
-    grid.rect(gp=gpar(col=NULL, fill="light grey"))
+    grid.rect(gp=gpar(col=NULL, fill=border.fill))
     # generate a viewport within the "top" viewport (vp) to represent the
     # parent viewport of the viewport we are "show"ing (v).
     # This is so that annotations at the edges of the
@@ -188,9 +194,9 @@ grid.show.viewport <- function(v, parent.layout=NULL, newpage=TRUE, vp=NULL) {
     w <- v$width
     h <- v$height
     pushViewport(v)
-    grid.rect(gp=gpar(col="blue", fill="light blue"))
+    grid.rect(gp=gpar(col=vp.col, fill=vp.fill))
     # represent the "native" scale
-    gp.red <- gpar(col="red")
+    gp.red <- gpar(col=scale.col)
     at <- grid.pretty(v$xscale)
     grid.xaxis(at=c(min(at), max(at)), gp=gp.red)
     at <- grid.pretty(v$yscale)
@@ -204,15 +210,15 @@ grid.show.viewport <- function(v, parent.layout=NULL, newpage=TRUE, vp=NULL) {
     popViewport()
     # annotate the location and dimensions of the viewport
     grid.lines(unit.c(x, x), unit.c(unit(0, "npc"), y),
-           gp=gpar(col="red", lty="dashed"))
+           gp=gpar(col=scale.col, lty="dashed"))
     grid.lines(unit.c(unit(0, "npc"), x), unit.c(y, y),
-           gp=gpar(col="red", lty="dashed"))
+           gp=gpar(col=scale.col, lty="dashed"))
     grid.text(as.character(x), gp=gp.red,
           just=c("centre", "top"),
           x=x, y=unit(-.05, "inches"))
     grid.text(as.character(y), gp=gp.red,
-          just=c("right", "centre"),
-          x=unit(-.05, "inches"), y=y)
+          just=c("bottom"),
+          x=unit(-.05, "inches"), y=y, rot=90)
     popViewport()
     if (!is.null(vp))
       popViewport()
