@@ -1,6 +1,6 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 1999-2002  the R Development Core Team
+ *  Copyright (C) 1999-2004  the R Development Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,6 +17,10 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <Defn.h>
 #include <R_ext/Random.h>	/* for the random number generation in
 				   samin() */
@@ -118,7 +122,7 @@ static void fmingr(int n, double *p, double *df, void *ex)
 		df[i] = (val1 - val2)/(2 * eps);
 #define DO_df_x 							\
 		if(!R_FINITE(df[i])) 					\
-		    error("non-finite finite-difference value [%d]", i);\
+		    error("non-finite finite-difference value [%d]", i+1);\
 		REAL(x)[i] = p[i] * (OS->parscale[i])
 
 		DO_df_x;
@@ -157,6 +161,36 @@ static void fmingr(int n, double *p, double *df, void *ex)
     }
 }
 
+static void genptry(int n, double *p, double *ptry, double scale, void *ex)
+{    
+    SEXP s, x;
+    int i;
+    OptStruct OS = (OptStruct) ex;
+    PROTECT_INDEX ipx;
+
+    if (!isNull(OS->R_gcall)) {  
+	/* user defined generation of candidate point */
+      	PROTECT(x = allocVector(REALSXP, n));
+	for (i = 0; i < n; i++) {
+	    if (!R_FINITE(p[i])) error("non-finite value supplied by optim");
+	    REAL(x)[i] = p[i] * (OS->parscale[i]);
+	}
+	SETCADR(OS->R_gcall, x);
+	PROTECT_WITH_INDEX(s = eval(OS->R_gcall, OS->R_env), &ipx);
+	REPROTECT(s = coerceVector(s, REALSXP), ipx);
+	if(LENGTH(s) != n)
+	    error("candidate point in optim evaluated to length %d not %d",
+		  LENGTH(s), n);
+	for (i = 0; i < n; i++)
+	    ptry[i] = REAL(s)[i] / (OS->parscale[i]);
+	UNPROTECT(2);
+    } 
+    else {  /* default Gaussian Markov kernel */
+        for (i = 0; i < n; i++)
+            ptry[i] = p[i] + scale * norm_rand();  /* new candidate point */
+    }
+}
+
 /* par fn gr method options */
 SEXP do_optim(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
@@ -221,15 +255,22 @@ SEXP do_optim(SEXP call, SEXP op, SEXP args, SEXP rho)
 	grcount = NA_INTEGER;
 
     }
-    else if (strcmp(tn, "SANN") == 0) {
-      tmax = asInteger(getListElement(options, "tmax"));
-      temp = asReal(getListElement(options, "temp"));
-      if (tmax == NA_INTEGER) error("tmax is not an integer");
-      samin (npar, dpar, &val, fminfn, maxit, tmax, temp, trace, (void *)OS);
-      for (i = 0; i < npar; i++)
-	  REAL(par)[i] = dpar[i] * (OS->parscale[i]);
-      fncount = maxit;
-      grcount = NA_INTEGER;
+    else if (strcmp(tn, "SANN") == 0) {	
+        tmax = asInteger(getListElement(options, "tmax"));
+        temp = asReal(getListElement(options, "temp"));
+        if (tmax == NA_INTEGER) error("tmax is not an integer");
+        if (!isNull(gr)) {
+            if (!isFunction(gr)) error("gr is not a function");
+                PROTECT(OS->R_gcall = lang2(gr, R_NilValue));
+        } else {
+	    PROTECT(OS->R_gcall = R_NilValue); /* for balance */
+        }
+        samin (npar, dpar, &val, fminfn, maxit, tmax, temp, trace, (void *)OS);
+        for (i = 0; i < npar; i++)
+            REAL(par)[i] = dpar[i] * (OS->parscale[i]);
+        fncount = maxit;
+        grcount = NA_INTEGER;
+        UNPROTECT(1);  /* OS->R_gcall */
 
     } else if (strcmp(tn, "BFGS") == 0) {
 	SEXP ndeps;
@@ -980,7 +1021,8 @@ void lbfgsb(int n, int m, double *x, double *l, double *u, int *nbd,
 
     *fail = 0;
     g = vect(n);
-    wa = vect(2*m*n+4*n+11*m*m+8*m);
+    /* this needs to be zeroed for snd in mainlb to be zeroed */
+    wa = (double *) S_alloc(2*m*n+4*n+11*m*m+8*m, sizeof(double));
     iwa = (int *) R_alloc(3*n, sizeof(int));
     strcpy(task, "START");
     while(1) {
@@ -1040,7 +1082,7 @@ void samin(int n, double *pb, double *yb, optimfn fminfn, int maxit,
    the function func).  Author: Adrian Trapletti
 */
 {
-    long i, j;
+    long j;
     int k, its, itdoc;
     double t, y, dy, ytry, scale;
     double *p, *dp, *ptry;
@@ -1058,15 +1100,12 @@ void samin(int n, double *pb, double *yb, optimfn fminfn, int maxit,
     }
     scale = 1.0/ti;
     its = itdoc = 1;
-    while (its < maxit) { /* cool down system */
+    while (its < maxit) {  /* cool down system */
 	t = ti/log((double)its + E1);  /* temperature annealing schedule */
 	k = 1;
 	while ((k <= tmax) && (its < maxit))  /* iterate at constant temperature */
 	{
-	    for (i = 0; i < n; i++)
-		dp[i] = scale * t * norm_rand();  /* random perturbation */
-	    for (i = 0; i < n; i++)
-		ptry[i] = p[i] + dp[i];  /* new candidate point */
+            genptry(n, p, ptry, scale * t, ex);  /* generate new candidate point */
 	    ytry = fminfn (n, ptry, ex);
 	    if (!R_FINITE(ytry)) ytry = big;
 	    dy = ytry - y;
